package com.gic.haoban.manage.service.service.hm.impl;

import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.manage.api.dto.hm.HmLinkDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkSearchQDTO;
import com.gic.haoban.manage.service.dao.mapper.hm.HmLinkMapper;
import com.gic.haoban.manage.service.entity.hm.TabHmLink;
import com.gic.haoban.manage.service.service.hm.HmLinkService;
import com.gic.haoban.manage.service.service.hm.HmLinkStoreService;
import com.gic.redis.data.util.RedisUtil;
import com.github.pagehelper.PageHelper;

@Service("hmLikService")
public class HmLinkServiceImpl implements HmLinkService {

    private static final Logger log = LoggerFactory.getLogger(HmLinkService.class);

    public static String[] PADDING = new String[] { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C",
            "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X",
            "Y", "Z", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s",
            "t", "u", "v", "w", "x", "y", "z" };

    @Autowired
    private HmLinkMapper linkMapper;
    @Autowired
    private HmLinkStoreService hmLinkStoreService;

    @Override
    public TabHmLink getByLinkId(String wxEnterpriseId, String enterprieId, Long linkId) {
        TabHmLink link = this.linkMapper.selectById(linkId);
        return link;
    }
    
    @Override
    public TabHmLink getByLinkShortCode(String enterpriseId, String shortCode) {
        TabHmLink link = this.linkMapper.selectByShortCode(enterpriseId, shortCode);
        return link;
    }

    @Override
    public TabHmLink getByLinkIdFormCache(String wxEnterpriseId, String enterprieId, Long linkId) {
        String key = "hm:link:" + linkId;
        TabHmLink link = RedisUtil.getCache(key, TabHmLink.class);
        if (link == null) {
            link = this.getByLinkId(wxEnterpriseId, enterprieId, linkId);
            if (link != null) {
                RedisUtil.setCache(key, link, 20 * 60L);
            }
        }
        return link;
    }

    @Override
    public List<HmLinkDTO> listForSms(String wxEnterpriseId, String enterpriseId, String linkName, Integer linkType) {
        List<TabHmLink> list = this.linkMapper.listForSms(wxEnterpriseId, enterpriseId, linkName, linkType);
        return EntityUtil.changeEntityListByJSON(HmLinkDTO.class, list);
    }

    @Override
    @Transactional
    public HmLinkDTO saveLink(HmLinkDTO dto, List<HmLinkStoreDTO> storeList) {
        // 新增
        Long linkId = dto.getLinkId();
        if (null == linkId) {
            linkId = UniqueIdUtils.uniqueLong();
            StringBuilder sb = new StringBuilder("LJ").append("0").append(dto.getLinkType()).append(linkId);
            dto.setLinkId(linkId);
            dto.setLinkCode(sb.toString());
            String shortCode = shortCode(dto.getEnterpriseId());
            dto.setShortCode(shortCode);
            dto.setStatusFlag(1);
            TabHmLink entity = EntityUtil.changeEntityByJSON(TabHmLink.class, dto);
            this.linkMapper.insert(entity);
        } else {
            TabHmLink entity = EntityUtil.changeEntityByJSON(TabHmLink.class, dto);
            this.linkMapper.update(entity);
        }
        this.hmLinkStoreService.saveStore(linkId, storeList);
        return dto;
    }

    @Override
    public Page<HmLinkDTO> listPage(HmLinkSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        String wxEnterpriseId = qdto.getWxEnterpriseId();
        String enterpriseId = qdto.getEnterpriseId();
        Date startDate = qdto.getStartDate();
        Date endDate = qdto.getEndDate();
        Integer linkType = qdto.getLinkType();
        List<TabHmLink> list = this.linkMapper.list(wxEnterpriseId, enterpriseId, startDate, endDate, linkType,
                qdto.getSearchParams());
        Page<HmLinkDTO> page = PageHelperUtils.changePageHelperToCurrentPage(list, HmLinkDTO.class);
        return page;
    }

    public String shortCode(String enterpriseId) {
        StringBuilder sb = new StringBuilder();
        String uuid = ToolUtil.randomUUID();
        int length = 5;
        int totalLength = 62;
        int interval = (int) (uuid.length() / length);
        String str;
        for (int i = 0; i < length; i++) {
            if (i == length - 1) {
                str = uuid.substring(length * interval);
            } else {
                str = uuid.substring(i * interval, i * interval + interval);
            }
            int x = Integer.parseInt(str, 16);
            sb.append(PADDING[x % totalLength]);
        }
        sb.append(PADDING[(int) (Math.random() * totalLength)]);
        String code =  sb.toString();
        TabHmLink link = this.linkMapper.selectByShortCode(enterpriseId, code) ;
        if(null == link) {
            return code ;
        }
        return shortCode(enterpriseId) ;
    }

}
